@@ -1,4 +1,5 @@
 import re
+import json
 from itertools import chain
 from pathlib import Path
 from collections import Counter
@@ -17,8 +18,11 @@ class NgramCorpus:
         MI, Xsq, Gsq, Dice, DeltaP21, DeltaP12, FisherExact
     ]
 
-    def __init__(self, corpus_reader, db_dir='dcctk.db/'):
+    def __init__(self, corpus_reader=None, corpus=None, db_dir='dcctk.db/'):
         self.corpus_reader = corpus_reader
+        self.corpus_mem = None
+        if isinstance(corpus, list):
+            self.corpus_mem = corpus
         self.n_subcorp = corpus_reader.n_subcorp
         self.database = {}
         self.pat_ch_chr = re.compile("[〇一-\u9fff㐀-\u4dbf豈-\ufaff]")
@@ -36,7 +40,9 @@ class NgramCorpus:
     def corpus(self):
         """Return a corpus as a generator
         """
-        return self.corpus_reader.get_corpus_as_gen()
+        if self.corpus_mem is None:
+            return self.corpus_reader.get_corpus_as_gen()
+        return self.corpus_mem
 
 
     def bigram_associations(self, subcorp_idx=None, chinese_only=True, 
@@ -70,10 +76,9 @@ class NgramCorpus:
 
     def freq_distr_ngrams(self, n, subcorp_idx=None, chinese_only=True):
         for k, v in self.get_ngrams(n, subcorp_idx).items():
-            k = k.decode('utf-8')
             if chinese_only: 
                 if any(not self.pat_ch_chr.search(ch) for ch in k): continue
-            yield (k, int(v))
+            yield (k, v)
 
     
     def get_ngrams(self, n, subcorp_idx=None):
@@ -103,9 +108,10 @@ class NgramCorpus:
         return sum(fq_lst)
 
 
-    def load(self, chr_fq=None):
+    def load(self, chr_fq:str=None):
         if chr_fq is not None:
-            self.chr_fq = chr_fq
+            with open(chr_fq, encoding="utf-8") as f:
+                self.chr_fq = json.load(f)
         else:
             print("[INFO] Counting char freq...")
             self._count_chr_fq()
@@ -135,6 +141,7 @@ class NgramCorpus:
         print(f'Counting {n}-grams...')
         if n == 2: self.subcorp_sizes = []
         pbar = tqdm(total=self.n_subcorp)
+        ngram = ('', '')
         for i, sc in enumerate(self.corpus):
             subcorp_size = 0
             subcorp_size_zh = 0
@@ -157,20 +164,20 @@ class NgramCorpus:
                     if n == 2:
                         # Count last character in sentence
                         ch = ngram[1]
-                        self.chr_fq.setdefault(ch, [0]*self.n_subcorp)[i] += 1 
-                        subcorp_size += 1
-                        if self.pat_ch_chr.search(ch):
-                            subcorp_size_zh += 1
+                        if ch != '':
+                            self.chr_fq.setdefault(ch, [0]*self.n_subcorp)[i] += 1 
+                            subcorp_size += 1
+                            if self.pat_ch_chr.search(ch):
+                                subcorp_size_zh += 1
                 
                 # Memory to disk
                 for k, v in db_tmp.items():
-                    vs = str(v)
                     if k not in db:
-                        db[k] = vs
-                        if i != 0: db_all[k] = vs
+                        db[k] = v
+                        if i != 0: db_all[k] = v
                     else:
-                        db[k] = str(int(db[k]) + v)
-                        if i != 0: db_all[k] = str(int(db_all[k]) + v)
+                        db[k] = db[k] + v
+                        if i != 0: db_all[k] = db_all[k] + v
             db.commit()
             db.close()
             if n == 2:
